package com.pyvio.acquirer.sdk;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.PropertyNamingStrategy;
import com.alibaba.fastjson.TypeReference;
import com.alibaba.fastjson.serializer.SerializeConfig;
import com.pyvio.acquirer.sdk.dto.req.*;
import com.pyvio.acquirer.sdk.dto.res.*;
import com.pyvio.acquirer.sdk.exception.PyvioErrorCode;
import com.pyvio.acquirer.sdk.exception.PyvioException;
import com.pyvio.acquirer.sdk.util.OkHttpClientSingleton;
import com.pyvio.acquirer.sdk.util.SignUtil;
import lombok.Builder;
import lombok.extern.slf4j.Slf4j;
import okhttp3.*;
import org.apache.commons.lang3.StringUtils;

import java.io.IOException;
import java.util.Objects;

@Builder
@Slf4j
public class PyvioPayClient {

    public static final String ACCESS_TOKEN_FILE = "oauth/api/v1/token";
    public static final String CHECKOUT_FILE = "acquirer-payment/payment/cashier/api/v1/checkout";
    public static final String PAYMENT_FILE = "acquirer-payment/payment/s2s/api/v1/pay";
    public static final String REFUND_FILE = "acquirer-payment/payment/api/v1/refund";
    public static final String CAPTURE_FILE = "acquirer-payment/payment/api/v1/capture";
    public static final String VOID_FILE = "acquirer-payment/payment/api/v1/void";
    public static final String QUERY_FILE = "acquirer-payment/payment/api/v1/query";
    public static final String LOGISTICS_UPLOAD_FILE = "acquirer-payment/logistics/api/v1/upload";

    private String appId;

    private String appSecret;

    private String host;

    private String privateKey;

    public PyvioPayClient(String appId, String appSecret, String host, String privateKey) {
        this.appId = appId;
        this.appSecret = appSecret;
        this.host = host;
        this.privateKey = privateKey;
    }

    public PyvioResponse<AccessTokenResDTO> accessToken(String requestId) {
        AccessTokenReqDTO accessTokenReqDTO = AccessTokenReqDTO.builder().appId(appId).appSecret(appSecret).grantType("client_credentials").build();
        String timestamp = String.valueOf(System.currentTimeMillis());
        SerializeConfig config = new SerializeConfig();
        config.propertyNamingStrategy = PropertyNamingStrategy.SnakeCase;
        String requestParams = JSON.toJSONString(accessTokenReqDTO, config);
        String sign = generateSign(appId, timestamp, requestParams);
        if (StringUtils.isBlank(sign)) throw new PyvioException(PyvioErrorCode.B_Sign_InvalidError);
        Request request = buildAccessTokenRequest(requestParams, sign, requestId, timestamp);
        try {
            log.info("PyvioPayClient.payment request:{}", requestParams);
            Response response = OkHttpClientSingleton.getClientInstance().newCall(request).execute();
            if (response.isSuccessful()) {
                if (Objects.nonNull(response.body())) {
                    String body = response.body().string();
                    log.info("PyvioPayClient.payment response:{}", body);
                    return JSONObject.parseObject(body, new TypeReference<>() {
                    });
                }
            }
            throw new PyvioException(PyvioErrorCode.B_Common_Error);
        } catch (IOException e) {
            log.error("PyvioPayClient.payment error, request:{}", requestParams, e);
            throw new PyvioException(PyvioErrorCode.B_Net_Error);
        }
    }



    public PyvioResponse<CheckoutResDTO> checkout(CheckoutReqDTO checkoutReqDTO, String token, String requestId) {
        String timestamp = String.valueOf(System.currentTimeMillis());
        SerializeConfig config = new SerializeConfig();
        config.propertyNamingStrategy = PropertyNamingStrategy.SnakeCase;
        String requestParams = JSON.toJSONString(checkoutReqDTO, config);
        String sign = generateSign(appId, timestamp, requestParams);
        if (StringUtils.isBlank(sign)) throw new PyvioException(PyvioErrorCode.B_Sign_InvalidError);
        Request request = buildRequest(requestParams, token, sign, requestId, timestamp, CHECKOUT_FILE);
        try {
            log.info("PyvioPayClient.checkout request:{}", requestParams);
            Response response = OkHttpClientSingleton.getClientInstance().newCall(request).execute();
            if (response.isSuccessful()) {
                if (Objects.nonNull(response.body())) {
                    String body = response.body().string();
                    log.info("PyvioPayClient.checkout response:{}", body);
                    return JSONObject.parseObject(body, new TypeReference<>() {
                    });
                }
            }
            throw new PyvioException(PyvioErrorCode.B_Common_Error);
        } catch (IOException e) {
            log.error("PyvioPayClient.checkout error, request:{}", requestParams, e);
            throw new PyvioException(PyvioErrorCode.B_Net_Error);
        }
    }


    public PyvioResponse<PaymentOrderResDTO> payment(PaymentPayReqDTO paymentPayReqDTO, String token, String requestId) {
        String timestamp = String.valueOf(System.currentTimeMillis());
        SerializeConfig config = new SerializeConfig();
        config.propertyNamingStrategy = PropertyNamingStrategy.SnakeCase;
        String requestParams = JSON.toJSONString(paymentPayReqDTO, config);
        String sign = generateSign(appId, timestamp, requestParams);
        if (StringUtils.isBlank(sign)) throw new PyvioException(PyvioErrorCode.B_Sign_InvalidError);
        Request request = buildRequest(requestParams, token, sign, requestId, timestamp, PAYMENT_FILE);
        try {
            log.info("PyvioPayClient.payment request:{}", requestParams);
            Response response = OkHttpClientSingleton.getClientInstance().newCall(request).execute();
            if (response.isSuccessful()) {
                if (Objects.nonNull(response.body())) {
                    String body = response.body().string();
                    log.info("PyvioPayClient.payment response:{}", body);
                    return JSONObject.parseObject(body, new TypeReference<>() {
                    });
                } else {
                    return PyvioResponse.success(PaymentOrderResDTO.builder().status("PENDING").build());
                }
            }
            throw new PyvioException(PyvioErrorCode.B_Common_Error);
        } catch (IOException e) {
            log.error("PyvioPayClient.payment error, request:{}", requestParams, e);
            if (e.getMessage().contains("Read timed out")) {
                return PyvioResponse.success(PaymentOrderResDTO.builder().status("PENDING").build());
            }
            throw new PyvioException(PyvioErrorCode.B_Net_Error);
        }
    }

    public PyvioResponse<RefundResDTO> refund(RefundReqDTO refundReqDTO, String token, String requestId) {
        String timestamp = String.valueOf(System.currentTimeMillis());
        SerializeConfig config = new SerializeConfig();
        config.propertyNamingStrategy = PropertyNamingStrategy.SnakeCase;
        String requestParams = JSON.toJSONString(refundReqDTO, config);
        String sign = generateSign(appId, timestamp, requestParams);
        if (StringUtils.isBlank(sign)) throw new PyvioException(PyvioErrorCode.B_Sign_InvalidError);
        Request request = buildRequest(requestParams, token, sign, requestId, timestamp, REFUND_FILE);
        try {
            log.info("PyvioPayClient.refund request:{}", requestParams);
            Response response = OkHttpClientSingleton.getClientInstance().newCall(request).execute();
            if (response.isSuccessful()) {
                if (Objects.nonNull(response.body())) {
                    String body = response.body().string();
                    log.info("PyvioPayClient.refund response:{}", body);
                    return JSONObject.parseObject(body, new TypeReference<>() {
                    });
                } else {
                    return PyvioResponse.success(RefundResDTO.builder().status("PENDING").build());
                }
            }
            throw new PyvioException(PyvioErrorCode.B_Common_Error);
        } catch (IOException e) {
            log.error("PyvioPayClient.refund error, request:{}", requestParams, e);
            if (e.getMessage().contains("Read timed out")) {
                return PyvioResponse.success(RefundResDTO.builder().status("PENDING").build());
            }
            throw new PyvioException(PyvioErrorCode.B_Net_Error);
        }
    }

    public PyvioResponse<PaymentOrderResDTO> query(QueryPaymentReqDTO queryPaymentReqDTO, String token, String requestId) {
        String timestamp = String.valueOf(System.currentTimeMillis());
        SerializeConfig config = new SerializeConfig();
        config.propertyNamingStrategy = PropertyNamingStrategy.SnakeCase;
        String requestParams = JSON.toJSONString(queryPaymentReqDTO, config);
        String sign = generateSign(appId, timestamp, requestParams);
        if (StringUtils.isBlank(sign)) throw new PyvioException(PyvioErrorCode.B_Sign_InvalidError);
        Request request = buildRequest(requestParams, token, sign, requestId, timestamp, QUERY_FILE);
        try {
            log.info("PyvioPayClient.query request:{}", requestParams);
            Response response = OkHttpClientSingleton.getClientInstance().newCall(request).execute();
            if (response.isSuccessful()) {
                if (Objects.nonNull(response.body())) {
                    String body = response.body().string();
                    log.info("PyvioPayClient.query response:{}", body);
                    return JSONObject.parseObject(body, new TypeReference<>() {
                    });
                }
            }
            throw new PyvioException(PyvioErrorCode.B_Common_Error);
        } catch (IOException e) {
            log.error("PyvioPayClient.query error, request:{}", requestParams, e);
            throw new PyvioException(PyvioErrorCode.B_Net_Error);
        }
    }

    public PyvioResponse<AuthCompleteResDTO> authCompleteCapture(AuthCompleteReqDTO authCompleteReqDTO, String token, String requestId) {
        String timestamp = String.valueOf(System.currentTimeMillis());
        SerializeConfig config = new SerializeConfig();
        config.propertyNamingStrategy = PropertyNamingStrategy.SnakeCase;
        String requestParams = JSON.toJSONString(authCompleteReqDTO, config);
        String sign = generateSign(appId, timestamp, requestParams);
        if (StringUtils.isBlank(sign)) throw new PyvioException(PyvioErrorCode.B_Sign_InvalidError);
        Request request = buildRequest(requestParams, token, sign, requestId, timestamp, CAPTURE_FILE);
        try {
            log.info("PyvioPayClient.authCompleteCapture request:{}", requestParams);
            Response response = OkHttpClientSingleton.getClientInstance().newCall(request).execute();
            if (response.isSuccessful()) {
                if (Objects.nonNull(response.body())) {
                    String body = response.body().string();
                    log.info("PyvioPayClient.authCompleteCapture response:{}", body);
                    return JSONObject.parseObject(body, new TypeReference<>() {
                    });
                } else {
                    return PyvioResponse.success(AuthCompleteResDTO.builder().status("PENDING").build());
                }
            }
            throw new PyvioException(PyvioErrorCode.B_Common_Error);
        } catch (IOException e) {
            log.error("PyvioPayClient.authCompleteCapture error, request:{}", requestParams, e);
            if (e.getMessage().contains("Read timed out")) {
                return PyvioResponse.success(AuthCompleteResDTO.builder().status("PENDING").build());
            }
            throw new PyvioException(PyvioErrorCode.B_Net_Error);
        }
    }

    public PyvioResponse<AuthCompleteResDTO> authCompleteVoid(AuthCompleteReqDTO authCompleteReqDTO, String token, String requestId) {
        String timestamp = String.valueOf(System.currentTimeMillis());
        SerializeConfig config = new SerializeConfig();
        config.propertyNamingStrategy = PropertyNamingStrategy.SnakeCase;
        String requestParams = JSON.toJSONString(authCompleteReqDTO, config);
        String sign = generateSign(appId, timestamp, requestParams);
        if (StringUtils.isBlank(sign)) throw new PyvioException(PyvioErrorCode.B_Sign_InvalidError);
        Request request = buildRequest(requestParams, token, sign, requestId, timestamp, VOID_FILE);
        try {
            log.info("PyvioPayClient.authCompleteVoid request:{}", requestParams);
            Response response = OkHttpClientSingleton.getClientInstance().newCall(request).execute();
            if (response.isSuccessful()) {
                if (Objects.nonNull(response.body())) {
                    String body = response.body().string();
                    log.info("PyvioPayClient.authCompleteVoid response:{}", body);
                    return JSONObject.parseObject(body, new TypeReference<>() {
                    });
                } else {
                    return PyvioResponse.success(AuthCompleteResDTO.builder().status("PENDING").build());
                }
            }
            throw new PyvioException(PyvioErrorCode.B_Common_Error);
        } catch (IOException e) {
            log.error("PyvioPayClient.authCompleteVoid error, request:{}", requestParams, e);
            if (e.getMessage().contains("Read timed out")) {
                return PyvioResponse.success(AuthCompleteResDTO.builder().status("PENDING").build());
            }
            throw new PyvioException(PyvioErrorCode.B_Net_Error);
        }
    }

    public PyvioResponse<LogisticsUploadResDTO> uploadLogistics(LogisticsUploadReqDTO logisticsUploadReqDTO, String token, String requestId) {
        String timestamp = String.valueOf(System.currentTimeMillis());
        SerializeConfig config = new SerializeConfig();
        config.propertyNamingStrategy = PropertyNamingStrategy.SnakeCase;
        String requestParams = JSON.toJSONString(logisticsUploadReqDTO, config);
        String sign = generateSign(appId, timestamp, requestParams);
        if (StringUtils.isBlank(sign)) throw new PyvioException(PyvioErrorCode.B_Sign_InvalidError);
        Request request = buildRequest(requestParams, token, sign, requestId, timestamp, LOGISTICS_UPLOAD_FILE);
        try {
            log.info("PyvioPayClient.uploadLogistics request:{}", requestParams);
            Response response = OkHttpClientSingleton.getClientInstance().newCall(request).execute();
            if (response.isSuccessful()) {
                if (Objects.nonNull(response.body())) {
                    String body = response.body().string();
                    log.info("PyvioPayClient.uploadLogistics response:{}", body);
                    return JSONObject.parseObject(body, new TypeReference<>() {
                    });
                }
            }
            throw new PyvioException(PyvioErrorCode.B_Common_Error);
        } catch (IOException e) {
            log.error("PyvioPayClient.uploadLogistics error, request:{}", requestParams, e);
            throw new PyvioException(PyvioErrorCode.B_Net_Error);
        }
    }

    private static PyvioResponse<?> sendRequest(String requestParams, Request request, String method) {
        try {
            log.info("PyvioPayClient.{} request:{}", method, requestParams);
            Response response = OkHttpClientSingleton.getClientInstance().newCall(request).execute();
            if (response.isSuccessful()) {
                if (Objects.nonNull(response.body())) {
                    String body = response.body().string();
                    log.info("PyvioPayClient.{} response:{}", method, body);
                    return JSONObject.parseObject(body, new TypeReference<>() {
                    });
                } else {
                    return PyvioResponse.success(PaymentOrderResDTO.builder().status("PENDING"));
                }
            } else {
                return PyvioResponse.fail(PyvioErrorCode.B_Common_Error);
            }
        } catch (IOException e) {
            log.error("PyvioPayClient.{} error, request:{}", method, requestParams, e);
            if (e.getMessage().contains("Read timed out")) {
                return PyvioResponse.success(PaymentOrderResDTO.builder().status("PENDING"));
            }
            return PyvioResponse.fail(PyvioErrorCode.B_Net_Error);
        }
    }

    private Request buildRequest(String requestParams, String token, String sign,
                                 String requestId, String timestamp, String path) {
        MediaType mediaType = MediaType.parse("application/json");
        RequestBody requestBody = RequestBody.create(mediaType, requestParams);
        return new Request.Builder()
                .post(requestBody)
                .url(buildUrl(host, path))
                .addHeader("Authorization", "Bearer " + token)
                .addHeader("Sign", sign)
                .addHeader("X-Timestamp", timestamp)
                .addHeader("Request-Id", requestId)
                .addHeader("Content-Type", "application/json")
                .build();
    }


    private Request buildAccessTokenRequest(String requestParams, String sign, String requestId, String timestamp) {
        MediaType mediaType = MediaType.parse("application/json");
        RequestBody requestBody = RequestBody.create(mediaType, requestParams);
        return new Request.Builder()
                .post(requestBody)
                .url(buildUrl(host, PyvioPayClient.ACCESS_TOKEN_FILE))
                .addHeader("Request-Id", requestId)
                .addHeader("Sign", sign)
                .addHeader("X-Timestamp", timestamp)
                .addHeader("Content-Type", "application/json")
                .build();
    }

    private String generateSign(String appId, String timestamp, String requestParams) {
        try {
            String signContent = appId + timestamp + requestParams;
            return SignUtil.sign(signContent, privateKey);
        } catch (Exception ignored) {}
        return "";
    }

    private String buildUrl(String domain, String file) {
        String protocol = "https://";
        String url = protocol + domain;
        if (url.endsWith("/")) {
            return url + file;
        } else {
            return url + "/" + file;
        }
    }
}
